# Generated by roxygen2: do not edit by hand

S3method("-",preference)
export(all_pred)
export(all_succ)
export(around)
export(between)
export(empty)
export(get_btg)
export(get_btg_dot)
export(get_hasse_diag)
export(hasse_pred)
export(hasse_succ)
export(high)
export(high_)
export(init_pred_succ)
export(is.base_pref)
export(is.complex_pref)
export(is.empty_pref)
export(is.preference)
export(layered)
export(low)
export(low_)
export(partial.eval.pref)
export(peval)
export(plot_btg)
export(plot_front)
export(pos)
export(pref.str)
export(psel)
export(psel.indices)
export(reverse)
export(show.pref)
export(show.query)
export(true)
export(true_)
exportMethods("&")
exportMethods("*")
exportMethods("+")
exportMethods("assoc.df<-")
exportMethods("|")
exportMethods(as.character)
exportMethods(as.expression)
exportMethods(assoc.df)
exportMethods(length)
import(igraph)
importFrom(Rcpp,cppFunction)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(RcppParallel,defaultNumThreads)
importFrom(dplyr,group_by)
importFrom(dplyr,is.grouped_df)
importFrom(graphics,par)
importFrom(graphics,segments)
importFrom(lazyeval,as.lazy)
importFrom(methods,new)
importFrom(utils,installed.packages)
useDynLib(rPref)
